--
-- SiloContainer NEW SCRIPT FOR LS2009
--
-- @Original-author Manuel L. (1.LS-Modschmiede)
-- @author Hoschiander (www.mein-ls.de)
-- @date 28/10/2009

SiloContainer = {};

--Vehicle.registerJointType("craneimplement");

function SiloContainer.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function SiloContainer:load(xmlFile)

	self.findTrailerRaycastCallback = Combine.findTrailerRaycastCallback;
	self.capacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.capacity"), 1000);

	self.refilling = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.options#refilling"), 1);

	self.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);
	
	self.trailerPosition = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.dischargeParticleSystems.dischargeParticleSystem#node"));
	self.grainUnloading = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.grainUnloading"), 10);

	self.isActive = false;
	self.pipeParticleActivated = false;
	self.sowingMachineFound = 0;

end;


function SiloContainer:getCorrectKey(key)
	if key < 256 then
		return string.upper(string.char(key));
	elseif key > 255 and key < 266 then
		return string.upper("KP"..string.char(key-208));
	else
		return "n/a";
	end;
end;

function SiloContainer:validateAttacherJoint(craneimplement, jointDesc, dt)
    return true;
end;

function SiloContainer:keyEvent(unicode, sym, modifier, isDown)
end;

function SiloContainer:update(dt)

	if self.attacherVehicle ~= nil  then
		local pipeParticleActivated = false;
		self.trailerFound = 0;
		self.sowingMachineFound = 0;
	    if self.fillLevel > 0 then
	        self.pipeParticleActivated = true;
	        self.trailerFound = 0;
	        local x,y,z = getWorldTranslation(self.trailerPosition);
	        raycastClosest(x, y, z, 0, -1, 0, "findTrailerRaycastCallback", 10, self);

	        local trailer = g_currentMission.objectToTrailer[self.trailerFound];
			local sowingMachine = nil;

			if self.sowingMachineFound ~= 0 then
				local implements = g_currentMission.attachables;
				for i=0, table.getn(implements) do
					if implements[i] ~= nil then
						if implements[i].rootNode == self.sowingMachineFound then
							sowingMachine = implements[i];
						end;
					end;
				end;
			end;

		if self.tipState == 0 then
	        if (self.trailerFound == 0 or not trailer:allowFillType(self.currentFillType)) and (self.sowingMachineFound == 0 or not sowingMachine:allowFillType(self.currentFillType)) then
	            self.pipeParticleActivated = false;
	        else
				local deltaLevel = self.grainUnloading;
				if sowingMachine ~= nil then
					deltaLevel = math.min(deltaLevel, sowingMachine.capacity - sowingMachine.fillLevel);
				elseif trailer ~= nil then
					deltaLevel = math.min(deltaLevel, trailer.capacity - trailer.fillLevel);
				else
					deltaLevel = 0;
				end;

				self.fillLevel = self.fillLevel-deltaLevel;
				if self.fillLevel <= 0.0 then
					deltaLevel = deltaLevel+self.fillLevel;
					self.fillLevel = 0.0;
					self.currentFillType = self.FILLTYPE_UNKNOWN;
					self.pipeParticleActivated = false;
				end;
				if deltaLevel == 0 then
					self.pipeParticleActivated = false;
				end;
				self:setFillLevel(self.fillLevel, self.currentFillType);
				if sowingMachine ~= nil then
					sowingMachine:setFillLevel(sowingMachine.fillLevel+deltaLevel, sowingMachine.self.currentFillType);
				elseif trailer ~= nil then
					trailer:setFillLevel(trailer.fillLevel+deltaLevel, self.currentFillType);
				end;
			end;
		else
		    self.pipeParticleActivated = true;
		end;
	    end;
	Utils.setEmittingState(self.dischargeParticleSystems[self.currentFillType], self.pipeParticleActivated);
	end;
end;

function SiloContainer:mouseEvent(posX, posY, isDown, isUp, button)
end;

function SiloContainer:draw()
	
--	print ("************************** self *******************************");
--	for k,v in pairs(self) do
--		print(tostring(k).."("..type(v)..") = "..tostring(v))
--	end;
--	print ("************************** Utils *******************************");
--	for k,v in pairs(Utils) do
--		print(tostring(k).."("..type(v)..") = "..tostring(v))
--	end;
end;

function SiloContainer:delete()
end;

function SiloContainer:onAttach(attacherVehicle)
end;

function SiloContainer:onDetach()
end;

function SiloContainer:findTrailerRaycastCallback(transformId, x, y, z, distance)

    self.trailerFound = 0;
	self.sowingMachineFound = 0;
    if getUserAttribute(transformId, "vehicleType") == 2 then
        self.trailerFound = transformId;
    end;

	if getUserAttribute(transformId, "sowingMachine") then
		self.sowingMachineFound = transformId;
	end;

    return false;
end;
